@@ -82,7 +82,7 @@ def SlicerPlotter(
     # inits
     la, ld = 0.7, 0.3 #ambient, diffuse
     dims = volume.dimensions()
-    data = volume.getPointArray()
+    data = volume.pointdata[0]
     rmin, rmax = volume.imagedata().GetScalarRange()
     if clamp:
         hdata, edg = np.histogram(data, bins=50)
@@ -252,7 +252,7 @@ def Slicer2d(volume, levels=(None, None), size=(900,900), bg='k9', zoom=1.2):
     if settings.vtk_version[0] == 9:
         axes=0
 
-    hist = cornerHistogram(volume.getPointArray(),
+    hist = cornerHistogram(volume.pointdata[0],
                            bins=25, logscale=1, pos=(0.02, 0.02), s=0.175,
                            c='dg', bg='k', alpha=1)
 
@@ -393,16 +393,16 @@ def RayCastPlotter(volume):
     )
     bum.status(volume.mode())
 
-    def CheckAbort(obj, event):
-        if obj.GetEventPending() != 0:
-            obj.SetAbortRender(1)
-
-    vp.window.AddObserver("AbortCheckEvent", CheckAbort)
+    # def CheckAbort(obj, event):
+    #     if obj.GetEventPending() != 0:
+    #         obj.SetAbortRender(1)
+    # vp.window.AddObserver("AbortCheckEvent", CheckAbort)
 
     # add histogram of scalar
-    plot = cornerHistogram(volume.getPointArray(),
+    plot = cornerHistogram(volume,
         bins=25, logscale=1, c=(.7,.7,.7), bg=(.7,.7,.7), pos=(0.78, 0.065),
         lines=True, dots=False,
+        nmax=3.1415e+06, # subsample otherwise is too slow
     )
 
     # xbins = np.linspace(smin, smax, 25)
@@ -419,11 +419,14 @@ def RayCastPlotter(volume):
     return vp
 
 
-def IsosurfaceBrowser(volume, c=None, alpha=1, lego=False, cmap='hot', pos=None):
+def IsosurfaceBrowser(volume, c=None, alpha=1, lego=False, cmap='hot', pos=None,
+                      delayed=False):
     """
     Generate a ``Plotter`` window for Volume isosurfacing using a slider.
     Returns the ``Plotter`` object.
 
+    Set delayed=True to delay slider update on mouse release.
+
     :Example:
         .. code-block:: python
 
@@ -485,15 +488,20 @@ def IsosurfaceBrowser(volume, c=None, alpha=1, lego=False, cmap='hot', pos=None)
                     value=threshold,
                     pos=sliderpos,
                     title=slidertitle,
-                    showValue=showval)
+                    showValue=showval,
+                    delayed=delayed,
+                    )
     return vp
 
 
 ##############################################################################
-def Browser(meshes, sliderpos=((0.55, 0.07),(0.96, 0.07)), c=None, prefix=""):
+def Browser(meshes, sliderpos=((0.55, 0.07),(0.96, 0.07)), c=None, prefix="",
+            # extras=(), #Not working
+            ):
     """
     Generate a ``Plotter`` window to browse a list of objects using a slider.
     Returns the ``Plotter`` object.
+
     """
 
     vp = settings.plotter_instance
@@ -518,14 +526,17 @@ def Browser(meshes, sliderpos=((0.55, 0.07),(0.96, 0.07)), c=None, prefix=""):
             tx = ak.name
         widget.GetRepresentation().SetTitleText(prefix+tx)
 
-    printc("Browser Mode", c="y", invert=1, end="")
-    printc(" loaded", len(meshes), "objects", c="y", bold=False)
+    # printc("Browser", c="y", invert=1, end="")
+    # printc(" loaded", len(meshes), "objects", c="y", bold=False)
 
     wid = vp.addSlider2D(sliderfunc, 0.5, len(meshes)-0.5,
                          pos=sliderpos, font='courier', c=c, showValue=False)
     wid.GetRepresentation().SetTitleHeight(0.020)
     sliderfunc(wid) # init call
 
+    # for e in extras:
+    #     vp.add(extras, render=False)
+
     return vp
 
 
@@ -533,11 +544,11 @@ class FreeHandCutPlotter(Plotter):
     """
     A Plotter derived class which edits polygonal meshes interactively.
     Can also be invoked from command line. E.g. with
-        
+
     ``vedo --edit https://vedo.embl.es/examples/data/porsche.ply``
 
     Usage:
-            
+
         - Left-click and hold to rotate
         - Right-click and move to draw line
         - Second right-click to stop drawing
