@@ -3,6 +3,7 @@ import vtk
 import vedo
 import vedo.colors as colors
 import vedo.utils as utils
+from deprecated import deprecated
 
 __doc__ = (
     """Base classes. Do not instantiate these."""
@@ -19,6 +20,105 @@ __all__ = [
            "streamLines",
 ]
 
+
+###############################################################################
+class _DataArrayHelper(object):
+    """Helper class to manage data associated to either
+    points (or vertices) and cells (or faces). Internal use only."""
+    def __init__(self, actor, association):
+        self.actor = actor
+        self.association = association
+
+    def __getitem__(self, key):
+        if self.association == 0:
+            data = self.actor._data.GetPointData()
+        else:
+            data = self.actor._data.GetCellData()
+
+        if isinstance(key, int):
+            key = data.GetArrayName(key)
+        arr = data.GetArray(key)
+        if not arr:
+            return None
+        return utils.vtk2numpy(arr)
+
+    def __setitem__(self, key, input_array):
+        if self.association == 0:
+            data = self.actor._data.GetPointData()
+            n = self.actor._data.GetNumberOfPoints()
+            self.actor._mapper.SetScalarModeToUsePointData()
+        else:
+            data = self.actor._data.GetCellData()
+            n = self.actor._data.GetNumberOfCells()
+            self.actor._mapper.SetScalarModeToUseCellData()
+
+        if len(input_array) != n:
+            colors.printc(f'Error in point/cell data: length of input {len(input_array)}'
+                          f' !=  {n} nr. of elements', c='r')
+            raise RuntimeError()
+
+        input_array = np.ascontiguousarray(input_array)
+        varr = utils.numpy2vtk(input_array, name=key)
+        data.AddArray(varr)
+
+        if len(input_array.shape)==1: # scalars
+            data.SetActiveScalars(key)
+        elif len(input_array.shape)==2 and input_array.shape[1] == 3: # vectors
+            if key == "Normals":
+                data.SetActiveNormals(key)
+            else:
+                data.SetActiveVectors(key)
+
+        return #####################
+
+    def keys(self):
+        if self.association == 0:
+            data = self.actor._data.GetPointData()
+        else:
+            data = self.actor._data.GetCellData()
+        arrnames = []
+        for i in range(data.GetNumberOfArrays()):
+            arrnames.append(data.GetArray(i).GetName())
+        return arrnames
+
+    def remove(self, key):
+        if self.association == 0:
+            self.actor._data.GetPointData().RemoveArray(key)
+        else:
+            self.actor._data.GetCellData().RemoveArray(key)
+
+    def rename(self, oldname, newname):
+        if self.association == 0:
+            varr = self.actor._data.GetPointData().GetArray(oldname)
+        else:
+            varr = self.actor._data.GetCellData().GetArray(oldname)
+        if varr:
+            varr.SetName(newname)
+        else:
+            colors.printc("Cannot rename non existing array:", oldname, "to:", newname, c='r')
+
+    def select(self, key):
+        if self.association == 0:
+            data = self.actor._data.GetPointData()
+            self.actor._mapper.SetScalarModeToUsePointData()
+        else:
+            data = self.actor._data.GetCellData()
+            self.actor._mapper.SetScalarModeToUseCellData()
+
+        if isinstance(key, int):
+            key = data.GetArrayName(key)
+        data.SetActiveScalars(key)
+
+        if hasattr(self.actor._mapper, 'SetArrayName'):
+            self.actor._mapper.SetArrayName(key)
+
+        if hasattr(self.actor._mapper, 'ScalarVisibilityOn'): # could be volume mapper
+            self.actor._mapper.ScalarVisibilityOn()
+
+    def print(self, **kwargs):
+        colors.printc(self.keys(), **kwargs)
+
+
 ###############################################################################
 class Base3DProp(object):
 
@@ -197,8 +297,7 @@ class Base3DProp(object):
         b, c, d = -axis * np.sin(anglerad / 2)
         aa, bb, cc, dd = a * a, b * b, c * c, d * d
         bc, ad, ac, ab, bd, cd = b * c, a * d, a * c, a * b, b * d, c * d
-        R = np.array(
-            [
+        R = np.array([
                 [aa + bb - cc - dd, 2 * (bc + ad), 2 * (bd - ac)],
                 [2 * (bc - ad), aa + cc - bb - dd, 2 * (cd + ab)],
                 [2 * (bd + ac), 2 * (cd - ab), aa + dd - bb - cc],
@@ -504,6 +603,7 @@ class Base3DProp(object):
                 pr = vtk.vtkProperty()
                 pr.DeepCopy(self.GetProperty())
                 bx.SetProperty(pr)
+                bx.property = pr
         bx.flat().lighting('off')
         bx.wireframe(not fill)
         return bx
@@ -631,6 +731,7 @@ class BaseActor(Base3DProp):
 
         self.flagText = None
         self._caption = None
+        self.property = None
 
 
     def mapper(self, newMapper=None):
@@ -665,7 +766,7 @@ class BaseActor(Base3DProp):
         return self.inputdata().GetNumberOfCells()
 
 
-    def points(self, pts=None, transformed=True, copy=False):
+    def points(self, pts=None, transformed=True):
         """
         Set/Get the vertex coordinates of a mesh or point cloud.
         Argument can be an index, a set of indices
@@ -673,8 +774,6 @@ class BaseActor(Base3DProp):
 
         :param bool transformed: if `False` ignore any previous transformation
             applied to the mesh.
-        :param bool copy: if `False` return the reference to the points
-            so that they can be modified in place, otherwise a copy is built.
         """
         if pts is None: ### getter
 
@@ -684,10 +783,7 @@ class BaseActor(Base3DProp):
                 vpts = self._data.GetPoints()
 
             if vpts:
-                if copy:
-                    return utils.vtk2numpy(vpts.GetData())
-                else:
-                    return utils.vtk2numpy(vpts.GetData())
+                return utils.vtk2numpy(vpts.GetData())
             else:
                 return np.array([])
 
@@ -701,7 +797,7 @@ class BaseActor(Base3DProp):
                 # assume plist is in the format [all_x, all_y, all_z]
                 pts = np.stack((pts[0], pts[1], pts[2]), axis=1)
             vpts = self._data.GetPoints()
-            vpts.SetData(utils.numpy2vtk(pts, dtype=np.float))
+            vpts.SetData(utils.numpy2vtk(pts, dtype=float))
             vpts.Modified()
             # reset mesh to identity matrix position/rotation:
             self.PokeMatrix(vtk.vtkMatrix4x4())
@@ -765,7 +861,8 @@ class BaseActor(Base3DProp):
             defcol = np.array(self.color())*255
             for i in range(cellData.GetNumberOfTuples()):
                 cellData.InsertTuple(i, defcol)
-            self.polydata(False).GetCellData().SetScalars(cellData)
+            self._data.GetCellData().AddArray(cellData)
+            self._data.GetCellData().SetActiveScalars('CellsWithinBoundsColor')
             self._mapper.ScalarVisibilityOn()
             flagcol = np.array(colors.getColor(c))*255
 
@@ -899,208 +996,100 @@ class BaseActor(Base3DProp):
         """
         return self.color(color, alpha)
 
+    @property
+    def pointdata(self):
+        """
+        Create and/or return a ``numpy.array`` associated to points (vertices).
+        A data array can be indexed either as a string or by an integer number.
+        E.g.:  ``myobj.pointdata["arrayname"]``
 
-    def getArrayNames(self):
-        """Get the existing arrays names as a dictionary"""
-        from vtk.numpy_interface import dataset_adapter
-        wrapped = dataset_adapter.WrapDataObject(self.GetMapper().GetInput())
-        return {"PointData":wrapped.PointData.keys(),
-                "CellData":wrapped.CellData.keys() }
+        Use:
 
+            ``myobj.pointdata.keys()`` to return the available data array names
+
+            ``myobj.pointdata.select(name)`` to make this array the active one
+
+            ``myobj.pointdata.remove(name)`` to remove this array
+        """
+        return _DataArrayHelper(self, 0)
+
+    @property
+    def celldata(self):
+        """
+        Create and/or return a ``numpy.array`` associated to cells (faces).
+        A data array can be indexed either as a string or by an integer number.
+        E.g.:  ``myobj.celldata["arrayname"]``
+
+        Use:
+
+            ``myobj.celldata.keys()`` to return the available data array names
+
+            ``myobj.celldata.select(name)`` to make this array the active one
+
+            ``myobj.celldata.remove(name)`` to remove this array
+        """
+        return _DataArrayHelper(self, 1)
+
+    @deprecated(reason=colors.red+"Please use myobj.pointdata[name] instead."+colors.reset)
     def getPointArray(self, name=0):
-        """
-        Return the point array content as a ``numpy.array``.
-        This can be identified either as a string or by an integer number.
-        """
-        data = self._data.GetPointData()
-        if isinstance(name, int):
-            name = data.GetArrayName(name)
-        arr = data.GetArray(name)
-        if not arr:
-            return None
-        return utils.vtk2numpy(arr)
+        """Deprecated. Use myobj.pointdata[name]` instead."""
+        return self.pointdata[name]
 
+    @deprecated(reason=colors.red+"Please use myobj.celldata[name] instead."+colors.reset)
     def getCellArray(self, name=0):
-        """
-        Return the cell array content as a ``numpy.array``.
-        This can be identified either as a string or by an integer number.
-        """
-        data = self._data.GetCellData()
-        if isinstance(name, int):
-            name = data.GetArrayName(name)
-        arr = data.GetArray(name)
-        if not arr:
-            return None
-        return utils.vtk2numpy(arr)
+        """Deprecated. Use myobj.celldata[name]` instead."""
+        return self.celldata[name]
 
+    @deprecated(reason=colors.red+"Please use myobj.pointdata[name] = myarr instead."+colors.reset)
+    def addPointArray(self, input_array, name):
+        """Deprecated. Use myobj.pointdata[name] = input_array` instead."""
+        self.pointdata[name] = input_array
+        return self
+
+    @deprecated(reason=colors.red+"Please use myobj.celldata[name] = myarr instead."+colors.reset)
+    def addCellArray(self, input_array, name):
+        """Deprecated. Use myobj.celldata[name] = input_array` instead."""
+        self.celldata[name] = input_array
+        return self
+
+    def mapCellsToPoints(self):
+        """
+        Interpolate cell data (i.e., data specified per cell or face)
+        into point data (i.e., data specified at each vertex).
+        The method of transformation is based on averaging the data values
+        of all cells using a particular point.
+        """
+        c2p = vtk.vtkCellDataToPointData()
+        c2p.SetInputData(self.inputdata())
+        c2p.Update()
+        self._mapper.SetScalarModeToUsePointData()
+        return self._update(c2p.GetOutput())
+
+    def mapPointsToCells(self):
+        """
+        Interpolate point data (i.e., data specified per point or vertex)
+        into cell data (i.e., data specified per cell).
+        The method of transformation is based on averaging the data values
+        of all points defining a particular cell.
+
+        |mesh_map2cell| |mesh_map2cell.py|_
+        """
+        p2c = vtk.vtkPointDataToCellData()
+        p2c.SetInputData(self.inputdata())
+        p2c.Update()
+        self._mapper.SetScalarModeToUseCellData()
+        return self._update(p2c.GetOutput())
 
     def addIDs(self, asfield=False):
-        """
-        Generate point and cell ids.
-
-        :param bool asfield: flag to control whether to generate scalar or field data.
-        """
+        """Generate point and cell ids."""
         ids = vtk.vtkIdFilter()
         ids.SetInputData(self._data)
         ids.PointIdsOn()
         ids.CellIdsOn()
-        if asfield:
-            ids.FieldDataOn()
-        else:
-            ids.FieldDataOff()
+        ids.FieldDataOff()
         ids.Update()
         return self._update(ids.GetOutput())
 
-    def selectPointArray(self, name):
-        """Make this point array the active one. Name can be a string or integer."""
-        data = self.inputdata().GetPointData()
-        arrname = name
-        if isinstance(name, int):
-            arrname = data.GetArrayName(name)
-        data.SetActiveScalars(arrname)
-        if hasattr(self._mapper, 'SetArrayName'):
-            self._mapper.SetArrayName(arrname)
-        self._mapper.SetScalarModeToUsePointData()
-        return self
-
-    def selectCellArray(self, name):
-        """Make this cell array the active one. Name can be a string or integer."""
-        data = self.inputdata().GetCellData()
-        arrname = name
-        if isinstance(name, int):
-            arrname = data.GetArrayName(name)
-        data.SetActiveScalars(arrname)
-        if hasattr(self._mapper, 'SetArrayName'):
-            self._mapper.SetArrayName(arrname)
-        self._mapper.SetScalarModeToUseCellData()
-        return self
-
-    def removePointArray(self, name):
-        """Reomve point array from object."""
-        self.inputdata().GetPointData().RemoveArray(name)
-        return self
-
-    def removeCellArray(self, name):
-        """Reomve cell array from object."""
-        self.inputdata().GetCellData().RemoveArray(name)
-        return self
-
-    def addPointArray(self, input_array, name):
-        """
-        Add point array and assign it a name.
-
-        |mesh_coloring| |mesh_coloring.py|_
-        """
-        data = self.inputdata()
-
-        if isinstance(input_array, vtk.vtkDataArray):
-            data.GetPointData().AddArray(input_array)
-            input_array.SetName(name)
-            data.GetPointData().SetActiveScalars(name)
-            self._mapper.ScalarVisibilityOn()
-            self._mapper.SetScalarRange(input_array.GetRange())
-            if hasattr(self._mapper, 'SetArrayName'):
-                self._mapper.SetArrayName(name)
-            self._mapper.SetScalarModeToUsePointData()
-            return self
-
-        if len(input_array) != data.GetNumberOfPoints():
-            colors.printc('Error in addPointArray(): Number of inputs != nr. of points',
-                          len(input_array), data.GetNumberOfPoints(), c='r')
-            raise RuntimeError()
-
-        nparr = np.ascontiguousarray(input_array)
-        if len(nparr.shape)==1: # scalars
-            varr = utils.numpy2vtk(nparr)
-            varr.SetName(name)
-            data.GetPointData().AddArray(varr)
-            data.GetPointData().SetActiveScalars(name)
-            self._mapper.SetScalarModeToUsePointData()
-            if hasattr(self._mapper, 'ScalarVisibilityOn'): # could be volume mapper
-                self._mapper.ScalarVisibilityOn()
-                self._mapper.SetScalarRange(varr.GetRange())
-
-        elif len(nparr.shape)==2: # vectors or higher dim ntuples
-            varr = vtk.vtkFloatArray()
-            varr.SetNumberOfComponents(nparr.shape[1])
-            varr.SetName(name)
-            for v in nparr:
-                varr.InsertNextTuple(v)
-            data.GetPointData().AddArray(varr)
-            if nparr.shape[1] == 3:
-                data.GetPointData().SetActiveVectors(name)
-        else:
-            colors.printc('Error in addPointArray(): cannot deal with shape:',
-                          nparr.shape, c='r')
-            return self
-
-        if hasattr(self._mapper, 'SetArrayName'):
-            self._mapper.SetArrayName(name)
-        self._mapper.SetScalarModeToUsePointData()
-
-        if name == "Normals":
-            data.GetPointData().SetActiveNormals(name)
-
-        return self
-
-    def addCellArray(self, input_array, name):
-        """
-        Add Cell array and assign it a name.
-
-        |mesh_coloring| |mesh_coloring.py|_
-        """
-        data = self.inputdata()
-
-        if isinstance(input_array, vtk.vtkDataArray):
-            data.GetCellData().AddArray(input_array)
-            input_array.SetName(name)
-            data.GetCellData().SetActiveScalars(name)
-            self._mapper.ScalarVisibilityOn()
-            self._mapper.SetScalarRange(input_array.GetRange())
-            if hasattr(self._mapper, 'SetArrayName'):
-                self._mapper.SetArrayName(name)
-            self._mapper.SetScalarModeToUseCellData()
-            return self
-            ###########
-
-        if len(input_array) != data.GetNumberOfCells():
-            colors.printc('Error in addCellArray(): Number of inputs != nr. of Cells',
-                          len(input_array), data.GetNumberOfCells(), c='r')
-            raise RuntimeError()
-
-        nparr = np.ascontiguousarray(input_array)
-        if len(nparr.shape)==1: # scalars
-            varr = utils.numpy2vtk(nparr)
-            varr.SetName(name)
-            data.GetCellData().AddArray(varr)
-            data.GetCellData().SetActiveScalars(name)
-            if hasattr(self._mapper, 'ScalarVisibilityOn'): # could be volume mapper
-                self._mapper.ScalarVisibilityOn()
-                self._mapper.SetScalarRange(varr.GetRange())
-
-        elif len(nparr.shape)==2: # vectors or higher dim ntuples
-            varr = vtk.vtkFloatArray()
-            varr.SetNumberOfComponents(nparr.shape[1])
-            varr.SetName(name)
-            for v in nparr:
-                varr.InsertNextTuple(v)
-            data.GetCellData().AddArray(varr)
-            if nparr.shape[1] == 3:
-                data.GetCellData().SetActiveVectors(name)
-        else:
-            colors.printc('Error in addCellArray(): cannot deal with shape:',
-                          nparr.shape, c='r')
-            return self
-
-        if hasattr(self._mapper, 'SetArrayName'):
-            self._mapper.SetArrayName(name)
-        self._mapper.SetScalarModeToUseCellData()
-
-        if name == "Normals":
-            data.GetCellData().SetActiveNormals(name)
-
-        return self
-
 
     def gradient(self, arrname=None, on='points', fast=False):
         """
@@ -1212,36 +1201,6 @@ class BaseActor(Base3DProp):
             vvecs = utils.vtk2numpy(vort.GetOutput().GetCellData().GetArray('Vorticity'))
         return vvecs
 
-
-    def mapCellsToPoints(self):
-        """
-        Transform cell data (i.e., data specified per cell)
-        into point data (i.e., data specified at each vertex).
-        The method of transformation is based on averaging the data values
-        of all cells using a particular point.
-        """
-        c2p = vtk.vtkCellDataToPointData()
-        c2p.SetInputData(self.inputdata())
-        c2p.Update()
-        self._mapper.SetScalarModeToUsePointData()
-        return self._update(c2p.GetOutput())
-
-    def mapPointsToCells(self):
-        """
-        Transform point data (i.e., data specified per point)
-        into cell data (i.e., data specified per cell).
-        The method of transformation is based on averaging the data values
-        of all points defining a particular cell.
-
-        |mesh_map2cell| |mesh_map2cell.py|_
-        """
-        p2c = vtk.vtkPointDataToCellData()
-        p2c.SetInputData(self.inputdata())
-        p2c.Update()
-        self._mapper.SetScalarModeToUseCellData()
-        return self._update(p2c.GetOutput())
-
-
     def addScalarBar(self,
                      title="",
                      pos=(0.8,0.05),
@@ -1458,20 +1417,20 @@ class BaseGrid(BaseActor):
         and you want -3 to show red and 1.5 violet and 6 green, then just set:
 
         ``volume.color(['red', 'violet', 'green'])``
-        
+
         :param list alpha: use a list to specify transparencies along the scalar range
         :param float vmin: force the min of the scalar range to be this value
         :param float vmax: force the max of the scalar range to be this value
         """
         # superseeds method in Points, Mesh
-        if vmin is None: 
+        if vmin is None:
             vmin, _ = self._data.GetScalarRange()
-        if vmax is None: 
+        if vmax is None:
             _, vmax = self._data.GetScalarRange()
         ctf = self.GetProperty().GetRGBTransferFunction()
         ctf.RemoveAllPoints()
         self._color = col
-        
+
         if utils.isSequence(col):
             if utils.isSequence(col[0]) and len(col[0])==2:
                 # user passing [(value1, color1), ...]
@@ -1522,9 +1481,9 @@ class BaseGrid(BaseActor):
         Then all cells below -5 will be completely transparent, cells with a scalar value of 35
         will get an opacity of 40% and above 123 alpha is set to 90%.
         """
-        if vmin is None: 
+        if vmin is None:
             vmin, _ = self._data.GetScalarRange()
-        if vmax is None: 
+        if vmax is None:
             _, vmax = self._data.GetScalarRange()
         otf = self.GetProperty().GetScalarOpacity()
         otf.RemoveAllPoints()
@@ -1654,7 +1613,7 @@ class BaseGrid(BaseActor):
         gf.Update()
 
         a = vedo.mesh.Mesh(gf.GetOutput()).lw(0.1).flat().lighting('ambient')
-        scalars = a.getPointArray()
+        scalars = a.pointdata[0]
         if scalars is None:
             print("Error in legosurface(): no scalars found!")
             return a
@@ -1747,7 +1706,8 @@ class BaseGrid(BaseActor):
                 p = ug.GetPoint(pointId)
                 signedDistance = ippd.EvaluateFunction(p)
                 signedDistances.InsertNextValue(signedDistance)
-            ug.GetPointData().SetScalars(signedDistances)
+            ug.GetPointData().AddArray(signedDistances)
+            ug.GetPointData().SetActiveScalars("SignedDistances")
             clipper = vtk.vtkClipDataSet()
             clipper.SetInputData(ug)
             clipper.SetInsideOut(not invert)
@@ -1760,9 +1720,9 @@ class BaseGrid(BaseActor):
             scalname = ug.GetCellData().GetScalars().GetName()
             if scalname: # not working
                 if self.useCells:
-                    self.selectCellArray(scalname)
+                    self.celldata.select(scalname)
                 else:
-                    self.selectPointArray(scalname)
+                    self.pointdata.select(scalname)
 
         self._update(cug)
         return self
@@ -1799,6 +1759,7 @@ class BaseGrid(BaseActor):
         pr = vtk.vtkProperty()
         pr.DeepCopy(self.GetProperty())
         tm_sel.SetProperty(pr)
+        tm_sel.property = pr
 
         #assign the same transformation to the copy
         tm_sel.SetOrigin(self.GetOrigin())
@@ -1823,7 +1784,7 @@ def probePoints(dataset, pts):
     and probes its scalars at the specified points in space.
 
     Note that a mask is also output with valid/invalid points which can be accessed
-    with `mesh.getPointArray('vtkValidPointMask')`.
+    with `mesh.pointdata['vtkValidPointMask']`.
     """
     if isinstance(pts, vedo.pointcloud.Points):
         pts = pts.points()
@@ -1861,7 +1822,7 @@ def probeLine(dataset, p1, p2, res=100):
     and probes its scalars along a line defined by 2 points `p1` and `p2`.
 
     Note that a mask is also output with valid/invalid points which can be accessed
-    with `mesh.getPointArray('vtkValidPointMask')`.
+    with `mesh.pointdata['vtkValidPointMask']`.
 
     :param int res: nr of points along the line
 
@@ -2169,8 +2130,6 @@ def streamLines(domain, probe,
         sta.mapper().SetScalarRange(scalarRange)
     return sta
 
-
-
 ###################################################################################
 # def extractCellsByType(obj, types=(7,)):    ### VTK9 only
 #     """Extract cells of a specified type.
@@ -2184,3 +2143,7 @@ def streamLines(domain, probe,
 #         ef.AddCellType(ct)
 #     ef.Update()
 #     return Mesh(ef.GetOutput())
+
+
+
+
