@@ -11,7 +11,7 @@ import vedo.docs as docs
 from vedo.colors import printc
 
 import vedo.settings as settings
-from vedo.settings import datadir, embedWindow
+from vedo.settings import dataurl, embedWindow
 
 from vedo.mesh import Mesh
 
@@ -39,9 +39,9 @@ Basic example:
     .. code-block:: python
 
         import dolfin
-        from vedo.dolfin import datadir, plot
+        from vedo.dolfin import dataurl, plot
 
-        mesh = dolfin.Mesh(datadir+"dolfin_fine.xml")
+        mesh = dolfin.Mesh(dataurl+"dolfin_fine.xml")
 
         plot(mesh)
 
@@ -115,7 +115,7 @@ __all__ = [
     "Text",
     "Text2D",
     "Latex",
-    "datadir",
+    "dataurl",
     "screenshot",
     "Video",
     "exportWindow",
@@ -756,7 +756,7 @@ class MeshActor(Mesh):
         if movedpts.shape[1] == 2: #2d
             movedpts = np.c_[movedpts, np.zeros(movedpts.shape[0])]
         self.polydata(False).GetPoints().SetData(numpy_to_vtk(np.ascontiguousarray(movedpts)))
-        self._polydata.GetPoints().Modified()
+        self.polydata(False).GetPoints().Modified()
 
 
 def MeshPoints(*inputobj, **options):
