@@ -777,12 +777,12 @@ def toNumpy(obj):
             adict['lines'] = obj.lines()
 
         adict['pointdata'] = []
-        for iname in obj.getArrayNames()['PointData']:
+        for iname in obj.pointdata.keys():
             if 'Normals' in iname.lower(): continue
             arr = poly.GetPointData().GetArray(iname)
             adict['pointdata'].append([utils.vtk2numpy(arr), iname])
         adict['celldata'] = []
-        for iname in obj.getArrayNames()['CellData']:
+        for iname in obj.celldata.keys():
             if 'Normals' in iname.lower(): continue
             arr = poly.GetCellData().GetArray(iname)
             adict['celldata'].append([utils.vtk2numpy(arr), iname])
@@ -1201,10 +1201,6 @@ def write(objct, fileoutput, binary=True):
         writer = vtk.vtkSimplePointsWriter()
     elif fr.endswith(".facet"):
         writer = vtk.vtkFacetWriter()
-    elif fr.endswith(".tif"):
-        writer = vtk.vtkTIFFWriter()
-        # print("GetCompression ", writer.GetCompression ())
-        writer.SetFileDimensionality(len(obj.GetDimensions()))
     elif fr.endswith(".vti"):
         writer = vtk.vtkXMLImageDataWriter()
     elif fr.endswith(".mhd"):
@@ -1217,6 +1213,10 @@ def write(objct, fileoutput, binary=True):
         writer = vtk.vtkJPEGWriter()
     elif fr.endswith(".bmp"):
         writer = vtk.vtkBMPWriter()
+    elif fr.endswith(".tif") or fr.endswith(".tiff"):
+        writer = vtk.vtkTIFFWriter()
+        # print("GetCompression ", writer.GetCompression()) # basically uncompressed..
+        writer.SetFileDimensionality(len(obj.GetDimensions()))
     elif fr.endswith(".npy") or fr.endswith(".npz"):
         if utils.isSequence(objct):
             objslist = objct
@@ -1349,7 +1349,7 @@ def writeTransform(inobj, filename='transform.mat', comment=''):
                       type(inobj), c='r')
     with open(filename,'w') as f:
         if comment:
-            f.write('#'+comment+'\n')
+            f.write('# '+comment+'\n')
         for i in range(4):
             f.write( str(M.GetElement(i,0))+' '+
                      str(M.GetElement(i,1))+' '+
@@ -1701,14 +1701,44 @@ def ask(*question, **kwarg):
     """
     Ask a question from command line. Return the answer as a string.
     See function `printc()` for the description of the options.
+
+    :param list options: a python list of possible answers to choose from.
+    :param str default: the default answer when just hitting return.
+
+    Example:
+
+        .. code-block:: python
+
+            import vedo
+            res = vedo.io.ask("Continue?", options=['Y','n'], default='Y', c='g')
+            print(res)
     """
     kwarg.update({'end': ' '})
     if 'invert' not in kwarg.keys():
         kwarg.update({'invert': True})
     if 'box' in kwarg.keys():
         kwarg.update({'box': ''})
-    colors.printc(*question, **kwarg)
+
+    options = kwarg.pop("options", [])
+    default = kwarg.pop("default", '')
+    if options:
+        opt = '['
+        for o in options:
+            opt += o + '/'
+        opt = opt[:-1]+']'
+        colors.printc(*question, opt, **kwarg)
+    else:
+        colors.printc(*question, **kwarg)
+
     resp = input()
+
+    if options:
+        if resp not in options:
+            if default and str(repr(resp))=="''":
+                return default
+            colors.printc("Please choose one option in:", opt, italic=True, bold=False)
+            kwarg["options"] = options
+            return ask(*question, **kwarg) # ask again
     return resp
 
 class Video:
