@@ -3,7 +3,7 @@ import os
 import vtk
 import vedo
 from vedo.colors import printc, getColor, colorMap
-from vedo.utils import isSequence, flatten, buildPolyData, numpy2vtk, vtk2numpy
+from vedo.utils import isSequence, flatten, mag, buildPolyData, numpy2vtk, vtk2numpy
 from vedo.pointcloud import Points
 from deprecated import deprecated
 
@@ -13,36 +13,48 @@ __all__ = ["Mesh", "merge"]
 
 
 ####################################################
-def merge(*meshs):
+def merge(*meshs, flag=False):
     """
-    Build a new mesh formed by the fusion of the polygonal meshes of the input objects.
-    Similar to Assembly, but in this case the input objects become a single mesh.
+    Build a new mesh formed by the fusion of the input polygonal Meshes (or Points).
 
-    .. hint:: |thinplate_grid.py|_ |value-iteration.py|_
+    Similar to Assembly, but in this case the input objects become a single mesh entity.
 
-        |thinplate_grid| |value-iteration|
+    To keep track of the original identities of the input mesh you can set flag.
+    In this case a point array of IDs is added to the merged output mesh.
+
+    .. hint:: |warp1.py|_ |value-iteration.py|_
+
+        |warp1| |value-iteration|
     """
-    acts = []
-    for a in flatten(meshs):
-        if a:
-            acts += [a]
+    acts = [a for a in flatten(meshs) if a]
 
     if not acts:
         return None
 
+    idarr = []
     polyapp = vtk.vtkAppendPolyData()
-    for a in acts:
-        if isinstance(a, vtk.vtkPolyData):
-            polyapp.AddInputData(a)
-        else:
-            polyapp.AddInputData(a.polydata())
+    for i, a in enumerate(acts):
+        try:
+            poly = a.polydata()
+        except:
+            # so a vtkPolydata can also be passed
+            poly = a
+        polyapp.AddInputData(poly)
+        if flag:
+            idarr += [i]*poly.GetNumberOfPoints()
     polyapp.Update()
+    mpoly = polyapp.GetOutput()
 
-    msh = Mesh(polyapp.GetOutput())
+    if flag:
+        varr = numpy2vtk(idarr, dtype=np.uint16, name="OriginalMeshID")
+        mpoly.GetPointData().AddArray(varr)
+
+    msh = Mesh(mpoly)
     if isinstance(acts[0], vtk.vtkActor):
         cprp = vtk.vtkProperty()
         cprp.DeepCopy(acts[0].GetProperty())
         msh.SetProperty(cprp)
+        msh.property = cprp
     return msh
 
 
@@ -100,6 +112,7 @@ class Mesh(Points):
             pr = vtk.vtkProperty()
             pr.DeepCopy(inputobj.GetProperty())
             self.SetProperty(pr)
+            self.property = pr
 
         elif isinstance(inputobj, vtk.vtkPolyData):
             if inputobj.GetNumberOfCells() == 0:
@@ -207,12 +220,6 @@ class Mesh(Points):
         self.property = self.GetProperty()
         self.property.SetInterpolationToPhong()
 
-        if vedo.settings.renderLinesAsTubes:
-            try:
-                self.property.RenderLinesAsTubesOn()
-            except:
-                pass
-
         # set the color by c or by scalar
         if self._data:
 
@@ -452,7 +459,7 @@ class Mesh(Points):
             tu.SetRepeat(repeat)
             tu.SetEdgeClamp(edgeClamp)
 
-        self.GetProperty().SetColor(1, 1, 1)
+        self.property.SetColor(1, 1, 1)
         self._mapper.ScalarVisibilityOff()
         self.SetTexture(tu)
 
@@ -463,7 +470,7 @@ class Mesh(Points):
             ugradm, vgradm = vedo.utils.mag2(ugrad), vedo.utils.mag2(vgrad)
             gradm = np.log(ugradm + vgradm)
             largegrad_ids = np.arange(len(grad))[gradm>seamThreshold*4]
-            uvmap = self.getPointArray(tname)
+            uvmap = self.pointdata[tname]
             # collapse triangles that have large gradient
             new_points = self.points(transformed=False)
             for f in self.faces():
@@ -547,9 +554,9 @@ class Mesh(Points):
         """Set mesh's representation as wireframe or solid surface.
         Same as `mesh.wireframe()`."""
         if value:
-            self.GetProperty().SetRepresentationToWireframe()
+            self.property.SetRepresentationToWireframe()
         else:
-            self.GetProperty().SetRepresentationToSurface()
+            self.property.SetRepresentationToSurface()
         return self
 
     def flat(self):
@@ -557,40 +564,29 @@ class Mesh(Points):
 
         |wikiphong|
         """
-        self.GetProperty().SetInterpolationToFlat()
+        self.property.SetInterpolationToFlat()
         return self
 
     def phong(self):
         """Set surface interpolation to Phong."""
-        self.GetProperty().SetInterpolationToPhong()
+        self.property.SetInterpolationToPhong()
         return self
 
     def backFaceCulling(self, value=True):
         """Set culling of polygons based on orientation
         of normal with respect to camera."""
-        self.GetProperty().SetBackfaceCulling(value)
+        self.property.SetBackfaceCulling(value)
+        return self
+
+    def renderLinesAsTubes(self, value=True):
+        self.property.SetRenderLinesAsTubes(value)
         return self
 
     def frontFaceCulling(self, value=True):
         """Set culling of polygons based on orientation of normal with respect to camera."""
-        self.GetProperty().SetFrontfaceCulling(value)
+        self.property.SetFrontfaceCulling(value)
         return self
 
-    # def depthSort(self, camera=None):
-    #     """Depth Sort"""
-    #     https://gitlab.kitware.com/vtk/vtk/-/issues/18173
-    #     depth_sort = vtk.vtkDepthSortPolyData()
-    #     depth_sort.SetInputData(self.polydata(False))
-    #     depth_sort.SetDirectionToBackToFront()
-    #     if camera is None:
-    #         if vedo.settings.plotter_instance and vedo.settings.plotter_instance.renderer:
-    #             cam = vedo.settings.plotter_instance.renderer.GetActiveCamera()
-    #             if cam:
-    #                 depth_sort.SetCamera(cam)
-    #     depth_sort.SortScalarsOff()
-    #     depth_sort.Update()
-    #     return self._update(depth_sort.GetOuput())
-
     def backColor(self, bc=None):
         """
         Set/get mesh's backface color.
@@ -602,7 +598,7 @@ class Mesh(Points):
                 return backProp.GetDiffuseColor()
             return self
 
-        if self.GetProperty().GetOpacity() < 1:
+        if self.property.GetOpacity() < 1:
             # printc("In backColor(): only active for alpha=1", c="y")
             return self
 
@@ -610,7 +606,7 @@ class Mesh(Points):
             backProp = vtk.vtkProperty()
 
         backProp.SetDiffuseColor(getColor(bc))
-        backProp.SetOpacity(self.GetProperty().GetOpacity())
+        backProp.SetOpacity(self.property.GetOpacity())
         self.SetBackfaceProperty(backProp)
         self._mapper.ScalarVisibilityOff()
         return self
@@ -623,13 +619,13 @@ class Mesh(Points):
         """Set/get width of mesh edges. Same as `lw()`."""
         if lw is not None:
             if lw == 0:
-                self.GetProperty().EdgeVisibilityOff()
-                self.GetProperty().SetRepresentationToSurface()
+                self.property.EdgeVisibilityOff()
+                self.property.SetRepresentationToSurface()
                 return self
-            self.GetProperty().EdgeVisibilityOn()
-            self.GetProperty().SetLineWidth(lw)
+            self.property.EdgeVisibilityOn()
+            self.property.SetLineWidth(lw)
         else:
-            return self.GetProperty().GetLineWidth()
+            return self.property.GetLineWidth()
         return self
 
     def lw(self, lineWidth=None):
@@ -639,14 +635,14 @@ class Mesh(Points):
     def lineColor(self, lc=None):
         """Set/get color of mesh edges. Same as `lc()`."""
         if lc is not None:
-            if "ireframe" in self.GetProperty().GetRepresentationAsString():
-                self.GetProperty().EdgeVisibilityOff()
-                self.color(lc)
-                return self
-            self.GetProperty().EdgeVisibilityOn()
-            self.GetProperty().SetEdgeColor(getColor(lc))
+#            if "ireframe" in self.property.GetRepresentationAsString():
+#                self.property.EdgeVisibilityOff()
+#                self.color(lc)
+#                return self
+            self.property.EdgeVisibilityOn()
+            self.property.SetEdgeColor(getColor(lc))
         else:
-            return self.GetProperty().GetEdgeColor()
+            return self.property.GetEdgeColor()
         return self
 
     def lc(self, lineColor=None):
@@ -920,8 +916,13 @@ class Mesh(Points):
         boundaryPoly.SetPoints(stripper.GetOutput().GetPoints())
         boundaryPoly.SetPolys(stripper.GetOutput().GetLines())
 
+        rev = vtk.vtkReverseSense()
+        rev.ReverseCellsOn()
+        rev.SetInputData(boundaryPoly)
+        rev.Update()
+
         tf = vtk.vtkTriangleFilter()
-        tf.SetInputData(boundaryPoly)
+        tf.SetInputData(rev.GetOutput())
         tf.Update()
 
         if returnCap:
@@ -937,7 +938,7 @@ class Mesh(Points):
             polyapp.AddInputData(poly)
             polyapp.AddInputData(tf.GetOutput())
             polyapp.Update()
-            return self._update(polyapp.GetOutput()).clean().phong()
+            return self._update(polyapp.GetOutput()).clean()
 
 
     def join(self, polys=True, reset=False):
@@ -1144,7 +1145,7 @@ class Mesh(Points):
     def addConnectivity(self):
         """
         Flag a mesh by connectivity: each disconnected region will receive a different Id.
-        You can access the array of ids through ``mesh.getPointArray("RegionId")``.
+        You can access the array of ids through ``mesh.pointdata["RegionId"]``.
         """
         cf = vtk.vtkConnectivityFilter()
         cf.SetInputData(self.polydata(False))
@@ -1152,7 +1153,7 @@ class Mesh(Points):
         cf.ColorRegionsOn()
         cf.Update()
         return self._update(cf.GetOutput())
-   
+
 
     def addElevationScalars(self, lowPoint=(0,0,0), highPoint=(0,0,1), vrange=(0,1)):
         """
@@ -1311,8 +1312,8 @@ class Mesh(Points):
 
     @deprecated(reason=vedo.colors.red+"Please use smooth()"+vedo.colors.reset)
     def smoothLaplacian(self, niter=15, relaxfact=0.1, edgeAngle=15, featureAngle=60, boundary=False):
-        return self.smooth(niter, passBand=0.1, edgeAngle=edgeAngle, boundary=boundary)  
-    
+        return self.smooth(niter, passBand=0.1, edgeAngle=edgeAngle, boundary=boundary)
+
     def smooth(self, niter=15, passBand=0.1, edgeAngle=15, featureAngle=60, boundary=False):
         """
         Adjust mesh point positions using the `Windowed Sinc` function interpolation kernel.
@@ -1392,7 +1393,7 @@ class Mesh(Points):
         else:
             vpoints = vtk.vtkPoints()
             pts = np.ascontiguousarray(pts)
-            vpoints.SetData(numpy2vtk(pts, dtype=np.float))
+            vpoints.SetData(numpy2vtk(pts, dtype=float))
             pointsPolydata = vtk.vtkPolyData()
             pointsPolydata.SetPoints(vpoints)
 
@@ -1403,7 +1404,7 @@ class Mesh(Points):
         sep.SetInsideOut(invert)
         sep.Update()
 
-        mask = Mesh(sep.GetOutput()).getPointArray(0).astype(np.bool)
+        mask = Mesh(sep.GetOutput()).pointdata[0].astype(np.bool)
         ids = np.array(range(len(pts)))[mask]
 
         if returnIds:
@@ -1419,7 +1420,7 @@ class Mesh(Points):
                    featureAngle=180,
                    returnPointIds=False,
                    returnCellIds=False,
-                   ):
+        ):
         """
         Return a ``Mesh`` that shows the boundary lines of an input mesh.
 
@@ -1531,12 +1532,12 @@ class Mesh(Points):
         gf.Update()
         return Mesh(gf.GetOutput()).lw(1)
 
-
-    def intersectWithLine(self, p0, p1=None, returnIds=False):
+    def intersectWithLine(self, p0, p1=None, returnIds=False, tol=0):
         """Return the list of points intersecting the mesh
         along the segment defined by two points `p0` and `p1`.
 
         :param bool returnIds: return the cell ids instead of point coords
+        :param float tol: tolerance/precision of the computation (0 = auto).
 
         :Example:
             .. code-block:: python
@@ -1556,32 +1557,30 @@ class Mesh(Points):
         if not self.line_locator:
             self.line_locator = vtk.vtkOBBTree()
             self.line_locator.SetDataSet(self.polydata())
+            if not tol:
+                tol = mag(np.asarray(p1)-np.asarray(p0))/10000
+            self.line_locator.SetTolerance(tol)
             self.line_locator.BuildLocator()
 
-        # t = vtk.mutable(tol)
-        # x = [0,0,0]
-        # pcoords = [0,0,0]
-        # subId = vtk.mutable(0)
-        # found = self.line_locator.IntersectWithLine(p0, p1, tol, t, x, pcoords, subId)
-        # if found:
-        #     return [x]
-        # else:
-        #     return []
+        intersectPoints = vtk.vtkPoints()
+        idlist = vtk.vtkIdList()
+        self.line_locator.IntersectWithLine(p0, p1, intersectPoints, idlist)
+        pts = []
+        for i in range(intersectPoints.GetNumberOfPoints()):
+            intersection = [0, 0, 0]
+            intersectPoints.GetPoint(i, intersection)
+            pts.append(intersection)
+        pts = np.array(pts)
 
         if returnIds:
-            idlist = vtk.vtkIdList()
-            self.line_locator.IntersectWithLine(p0, p1, None, idlist)
-            return [idlist.GetId(i) for i in range(idlist.GetNumberOfIds())]
-
+            pts_ids = []
+            for i in range(idlist.GetNumberOfIds()):
+                cid = idlist.GetId(i)
+                pts_ids.append([pts[i], cid])
+            return pts_ids
         else:
-            intersectPoints = vtk.vtkPoints()
-            self.line_locator.IntersectWithLine(p0, p1, intersectPoints, None)
-            pts = []
-            for i in range(intersectPoints.GetNumberOfPoints()):
-                intersection = [0, 0, 0]
-                intersectPoints.GetPoint(i, intersection)
-                pts.append(intersection)
-            return np.array(pts)
+            return pts
+
 
     def silhouette(self, direction=None, borderEdges=True, featureAngle=False):
         """
@@ -1790,6 +1789,7 @@ class Mesh(Points):
             return self
         else:
             rf = vtk.vtkRotationalExtrusionFilter()
+            # rf = vtk.vtkLinearExtrusionFilter()
             rf.SetInputData(self.polydata(False)) #must not be transformed
             rf.SetResolution(res)
             rf.SetCapping(cap)
@@ -1799,8 +1799,9 @@ class Mesh(Points):
             rf.Update()
             m = Mesh(rf.GetOutput(), c=self.c(), alpha=self.alpha())
             prop = vtk.vtkProperty()
-            prop.DeepCopy(self.GetProperty())
+            prop.DeepCopy(self.property)
             m.SetProperty(prop)
+            m.property = prop
             # assign the same transformation
             m.SetOrigin(self.GetOrigin())
             m.SetScale(self.GetScale())
@@ -1828,7 +1829,7 @@ class Mesh(Points):
         a = Mesh(cf.GetOutput())
         alist = []
 
-        for t in range(max(a.getPointArray("RegionId")) + 1):
+        for t in range(max(a.pointdata["RegionId"]) + 1):
             if t == maxdepth:
                 break
             suba = a.clone().threshold("RegionId", t - 0.1, t + 0.1)
@@ -1859,8 +1860,9 @@ class Mesh(Points):
         conn.Update()
         m = Mesh(conn.GetOutput())
         pr = vtk.vtkProperty()
-        pr.DeepCopy(self.GetProperty())
+        pr.DeepCopy(self.property)
         m.SetProperty(pr)
+        m.property = pr
         # assign the same transformation
         m.SetOrigin(self.GetOrigin())
         m.SetScale(self.GetScale())
@@ -1925,43 +1927,52 @@ class Mesh(Points):
         Takes as input a polygonal mesh and performs a single source
         shortest path calculation.
 
-        :param start: start vertex index or close point `[x,y,z]`
-        :type start: int, list
-        :param end: end vertex index or close point `[x,y,z]`
-        :type start: int, list
+        :param int,list start: start vertex index or close point `[x,y,z]`
+        :param int,list end: end vertex index or close point `[x,y,z]`
 
         |geodesic| |geodesic.py|_
         """
-        dijkstra = vtk.vtkDijkstraGraphGeodesicPath()
-
         if isSequence(start):
             cc = self.points()
             pa = Points(cc)
-            start = pa.closestPoint(start, returnIds=True)
-            end = pa.closestPoint(end, returnIds=True)
-            dijkstra.SetInputData(pa.polydata())
-        else:
-            dijkstra.SetInputData(self.polydata())
+            start = pa.closestPoint(start, returnPointId=True)
+            end   = pa.closestPoint(end,   returnPointId=True)
 
-        dijkstra.SetStartVertex(start)
-        dijkstra.SetEndVertex(end)
+        dijkstra = vtk.vtkDijkstraGraphGeodesicPath()
+        dijkstra.SetInputData(self.polydata())
+        dijkstra.SetStartVertex(end) # inverted in vtk
+        dijkstra.SetEndVertex(start)
         dijkstra.Update()
 
         weights = vtk.vtkDoubleArray()
         dijkstra.GetCumulativeWeights(weights)
 
+        idlist = dijkstra.GetIdList()
+        ids = [idlist.GetId(i) for i in range(idlist.GetNumberOfIds())]
+
         length = weights.GetMaxId() + 1
         arr = np.zeros(length)
         for i in range(length):
             arr[i] = weights.GetTuple(i)[0]
 
-        dmesh = Mesh(dijkstra.GetOutput())
+        poly = dijkstra.GetOutput()
+
+        vdata = numpy2vtk(arr)
+        vdata.SetName("CumulativeWeights")
+        poly.GetPointData().AddArray(vdata)
+
+        vdata2 = numpy2vtk(ids, dtype=np.uint)
+        vdata2.SetName("VertexIDs")
+        poly.GetPointData().AddArray(vdata2)
+        poly.GetPointData().Modified()
+
+        dmesh = Mesh(poly, c='k')
         prop = vtk.vtkProperty()
-        prop.DeepCopy(self.GetProperty())
+        prop.DeepCopy(self.property)
         prop.SetLineWidth(3)
         prop.SetOpacity(1)
         dmesh.SetProperty(prop)
-        dmesh.info["CumulativeWeights"] = arr
+        dmesh.property = prop
         dmesh.name = "geodesicLine"
         return dmesh
 
