@@ -446,12 +446,13 @@ def histogram(*args, **kwargs):
     elif len(args) == 1:
 
         if isinstance(args[0], vedo.Volume):
-            data = args[0].getPointArray()
+            data = args[0].pointdata[0]
         elif isinstance(args[0], vedo.Points):
-            if args[0].getPointArray():
-                data = args[0].getPointArray().ravel()
+            pd0 = args[0].pointdata[0]
+            if pd0:
+                data = pd0.ravel()
             else:
-                data = args[0].getCellArray().ravel()
+                data = args[0].celldata[0].ravel()
         else:
             data = np.array(args[0])
 
@@ -674,7 +675,6 @@ def _plotxy(
         ylim = format.ylim
         aspect = format.aspect
         pad = format.pad
-        axes = 0
         title = ""
         xtitle = ""
         ytitle = ""
@@ -863,7 +863,7 @@ def _plotxy(
         axes["xrange"] = (x0lim, x1lim)
         axes["yrange"] = (y0lim, y1lim)
         axes["zrange"] = (0, 0)
-        axes["c"] = "k"
+        # axes["c"] = "k"
         axes["yUseBounds"] = True
         axs = addons.Axes(**axes)
         axs.name = "axes"
@@ -1980,7 +1980,7 @@ def _histogramHexBin(
     zs = [[0.0]] * len(values)
     values = np.append(values, zs, axis=1)
 
-    pointsPolydata.GetPoints().SetData(utils.numpy2vtk(values, dtype=np.float))
+    pointsPolydata.GetPoints().SetData(utils.numpy2vtk(values, dtype=float))
     cloud = Mesh(pointsPolydata)
 
     col = None
@@ -2345,7 +2345,6 @@ def quiver(
     headLength=0.25,
     headWidth=0.2,
     fill=True,
-    scale=1,
 ):
     """
     Quiver Plot, display `vectors` at `points` locations.
@@ -2357,7 +2356,6 @@ def quiver(
     :param float headLength: fractional head length
     :param float headWidth: fractional head width
     :param bool fill: if False only generate the outline
-    :param float scale: apply a rescaling factor to the length
 
     |quiver| |quiver.py|_
     """
@@ -2379,7 +2377,6 @@ def quiver(
         headLength=headLength,
         headWidth=headWidth,
         fill=fill,
-        scale=scale,
         alpha=alpha,
     )
     arrs2d.pickable(False)
@@ -2823,6 +2820,7 @@ def cornerHistogram(
     s=0.175,
     lines=True,
     dots=False,
+    nmax=None,
 ):
     """
     Build a histogram from a list of values in n bins.
@@ -2830,6 +2828,8 @@ def cornerHistogram(
 
     Use *vrange* to restrict the range of the histogram.
 
+    :param int nmax: limit the sampling to this max nr of entries
+
     Use `pos` to assign its position:
         - 1, topleft,
         - 2, topright,
@@ -2840,7 +2840,14 @@ def cornerHistogram(
     if hasattr(values, '_data'):
         values = utils.vtk2numpy(values._data.GetPointData().GetScalars())
 
+    n = values.shape[0]
+    if nmax and nmax < n:
+        # subsample:
+        idxs = np.linspace(0, n, num=int(nmax), endpoint=False).astype(int)
+        values = values[idxs]
+
     fs, edges = np.histogram(values, bins=bins, range=vrange)
+
     if minbin:
         fs = fs[minbin:-1]
     if logscale:
