@@ -19,9 +19,6 @@ General settings.
     # Recompute vertex and cell normals
     computeNormals = None
 
-    # Automatically reset the range of the active scalars
-    autoResetScalarRange = True
-
     # Allow to continously interact with scene during interactive() execution
     allowInteraction = True
 
@@ -45,9 +42,6 @@ General settings.
     # Enable / disable color printing by printc()
     enablePrintColor = True
 
-    # Qt embedding
-    usingQt = False
-
     # Wrap lines in tubes
     renderLinesAsTubes = False
 
@@ -69,8 +63,8 @@ General settings.
     # Turn on/off rendering of translucent material with depth peeling technique.
     useDepthPeeling = False
     alphaBitPlanes  = True  # options only active if useDepthPeeling=True
-    multiSamples    = 0     # force to not pick a framebuffer with a multisample buffer
-    maxNumberOfPeels= 8     # maximum number of rendering passes
+    multiSamples    = 8     # force to not pick a framebuffer with a multisample buffer
+    maxNumberOfPeels= 4     # maximum number of rendering passes
     occlusionRatio  = 0.0   # occlusion ratio, 0 = exact image.
 
     # Turn on/off nvidia FXAA post-process anti-aliasing, if supported.
@@ -102,7 +96,7 @@ General settings.
     # BOTRIGHT 3 (row 0 bottom, col 0 rhs) BOTLEFT  4 (row 0 bottom, col 0 lhs)
     # LEFTTOP  5 (row 0 lhs, col 0 top)    RIGHTTOP 6 (row 0 rhs, col 0 top)
     # RIGHTBOT 7 (row 0 rhs, col 0 bottom) LEFTBOT  8 (row 0 lhs, col 0 bottom)
-    tiffOrientationType = 4
+    tiffOrientationType = 1
 
     # AnnotatedCube axis type nr. 5 options:
     annotatedCubeColor      = (0.75, 0.75, 0.75)
@@ -132,6 +126,8 @@ Usage example:
     Cube().color('green').show()
 """
 import os, vtk
+import numpy as np
+import warnings
 
 __all__ = ['dataurl', 'embedWindow']
 
@@ -164,9 +160,6 @@ screeshotLargeImage = False
 # Recompute vertex and cell normals
 computeNormals = None
 
-# Automatic resetting the range of the active scalars
-autoResetScalarRange = True
-
 # Allow to continously interact with scene during interactor.Start() execution
 allowInteraction = True
 
@@ -184,10 +177,8 @@ rendererFrameAlpha = 0.5
 rendererFrameWidth = 0.5
 rendererFramePadding = 0.0001
 
-# Qt embedding
-usingQt = False
-
 # Wrap lines in tubes
+# renderPointsAsSpheres has become mesh.renderPointsAsSpheres(True)
 renderLinesAsTubes = False
 
 # Remove hidden lines when in wireframe mode
@@ -213,9 +204,9 @@ if vtk_version[0] >= 9:
     if "Windows" in sys_platform:
         useDepthPeeling = True
 # only relevant if depthpeeling is on
-alphaBitPlanes = 1
-maxNumberOfPeels= 4
-occlusionRatio  = 0.1
+alphaBitPlanes   = 1
+maxNumberOfPeels = 4
+occlusionRatio   = 0.1
 
 # Turn on/off nvidia FXAA anti-aliasing, if supported.
 useFXAA = False  # either True or False
@@ -535,7 +526,7 @@ def embedWindow(backend='ipyvtk', verbose=True):
             import k3d
             if k3d._version.version_info != (2, 7, 4):
                 print('Warning: only k3d version 2.7.4 is currently supported')
-#                print('> pip install k3d==2.7.4')
+                # print('> pip install k3d==2.7.4')
             
         except:
             notebookBackend = None
@@ -605,7 +596,7 @@ def _init():
         fonts.append(f.split(".")[0])
     fonts = list(sorted(fonts))
 
-    import warnings
     warnings.simplefilter(action="ignore", category=FutureWarning)
+    np.warnings.filterwarnings('ignore', category=np.VisibleDeprecationWarning) 
 
     embedWindow()
