@@ -1,5 +1,6 @@
 import vtk
 from vtk.util.numpy_support import numpy_to_vtk, vtk_to_numpy, numpy_to_vtkIdTypeArray
+# from vtk.util.numpy_support import get_vtk_array_type, get_vtk_to_numpy_typemap
 import numpy as np
 import vedo
 from vedo.colors import printc
@@ -253,17 +254,22 @@ class dotdict(dict):
 
 ###########################################################
 def numpy2vtk(arr, dtype=None, deep=True, name=""):
-    """Convert a numpy array into a vtkDataArray"""
+    """Convert a numpy array into a vtkDataArray.
+    Use dtype='id' for vtkIdTypeArray objects."""
+    # https://github.com/Kitware/VTK/blob/master/Wrapping/Python/vtkmodules/util/numpy_support.py
     if arr is None:
         return None
-    arr = np.ascontiguousarray(arr)
-    if dtype is not None and dtype!='id':
-        arr = arr.astype(dtype)
 
-    if dtype and dtype=='id':
+    arr = np.ascontiguousarray(arr)
+
+    if dtype=='id':
         varr = numpy_to_vtkIdTypeArray(arr.astype(np.int64), deep=deep)
+    elif dtype:
+        varr = numpy_to_vtk(arr.astype(dtype), deep=deep)
     else:
+        # let numpy_to_vtk() decide what is best type based on arr type
         varr = numpy_to_vtk(arr, deep=deep)
+
     if name:
         varr.SetName(name)
     return varr
@@ -293,13 +299,12 @@ def geometry(obj, extent=None):
 
     :param list extent: set a `[xmin,xmax, ymin,ymax, zmin,zmax]` bounding box to clip data.
     """
-    from vedo.mesh import Mesh
     gf = vtk.vtkGeometryFilter()
     gf.SetInputData(obj)
     if extent is not None:
         gf.SetExtent(extent)
     gf.Update()
-    return Mesh(gf.GetOutput())
+    return vedo.Mesh(gf.GetOutput())
 
 
 def buildPolyData(vertices, faces=None, lines=None, indexOffset=0, fast=True, tetras=False):
@@ -335,7 +340,7 @@ def buildPolyData(vertices, faces=None, lines=None, indexOffset=0, fast=True, te
 
     sourcePoints = vtk.vtkPoints()
     # sourcePoints.SetData(numpy_to_vtk(np.ascontiguousarray(vertices), deep=True))
-    sourcePoints.SetData(numpy2vtk(vertices, dtype=np.float))
+    sourcePoints.SetData(numpy2vtk(vertices, dtype=float))
     poly.SetPoints(sourcePoints)
 
     if lines is not None:
@@ -370,8 +375,11 @@ def buildPolyData(vertices, faces=None, lines=None, indexOffset=0, fast=True, te
 
     # faces exist
     sourcePolygons = vtk.vtkCellArray()
-    faces = np.array(faces)
-    if len(faces.shape) == 2 and indexOffset==0 and fast:
+
+    # try it anyway: in case it's not uniform np.ndim will be 1
+    faces = np.asarray(faces)
+
+    if np.ndim(faces) == 2 and indexOffset==0 and fast:
         #################### all faces are composed of equal nr of vtxs, FAST
 
         ast = np.int32
@@ -618,9 +626,9 @@ def linInterpolate(x, rangeX, rangeY):
     |linInterpolate| |linInterpolate.py|_
     """
     if isSequence(x):
-        x = np.array(x)
-        x0, x1 = np.array(rangeX)
-        y0, y1 = np.array(rangeY)
+        x = np.asarray(x)
+        x0, x1 = np.asarray(rangeX)
+        y0, y1 = np.asarray(rangeY)
         # if len(np.unique([x.shape, x0.shape, x1.shape, y1.shape]))>1:
         #     printc("Error in linInterpolate(): mismatch in input shapes.", c='r')
         #     raise RuntimeError()
@@ -688,7 +696,7 @@ def roundToDigit(x, p):
     """Round a real number to the specified number of significant digits."""
     if not x:
         return x
-    k = np.int(np.floor(np.log10(np.abs(x)))) + (p-1)
+    k = int(np.floor(np.log10(np.abs(x)))) + (p-1)
     r = np.around(x, -k)
     if int(r) == r:
         return int(r)
@@ -844,9 +852,9 @@ def grep(filename, tag, firstOccurrence=False):
 
     try:
         afile = open(filename, "r")
-    except:
-        print("Error in utils.grep(): cannot open file", filename)
-        raise RuntimeError()
+    except FileNotFoundError:
+        printc("Error in utils.grep(): cannot open file", filename, c='r')
+        return []
 
     content = []
     for line in afile:
@@ -979,19 +987,23 @@ def printInfo(obj):
         if ptdata.GetNumberOfArrays() + cldata.GetNumberOfArrays():
 
             arrtypes = dict()
-            arrtypes[vtk.VTK_UNSIGNED_CHAR] = "UNSIGNED_CHAR"
-            arrtypes[vtk.VTK_SIGNED_CHAR]   = "SIGNED_CHAR"
-            arrtypes[vtk.VTK_UNSIGNED_INT]  = "UNSIGNED_INT"
-            arrtypes[vtk.VTK_INT]           = "INT"
-            arrtypes[vtk.VTK_CHAR]          = "CHAR"
-            arrtypes[vtk.VTK_SHORT]         = "SHORT"
-            arrtypes[vtk.VTK_LONG]          = "LONG"
-            arrtypes[vtk.VTK_ID_TYPE]       = "ID"
-            arrtypes[vtk.VTK_FLOAT]         = "FLOAT"
-            arrtypes[vtk.VTK_DOUBLE]        = "DOUBLE"
+            arrtypes[vtk.VTK_UNSIGNED_CHAR] = ("UNSIGNED_CHAR",  "np.uint8")
+            arrtypes[vtk.VTK_UNSIGNED_SHORT]= ("UNSIGNED_SHORT", "np.uint16")
+            arrtypes[vtk.VTK_UNSIGNED_INT]  = ("UNSIGNED_INT",   "np.uint32")
+            arrtypes[vtk.VTK_UNSIGNED_LONG_LONG] = ("UNSIGNED_LONG_LONG", "np.uint64")
+            arrtypes[vtk.VTK_CHAR]          = ("CHAR",           "np.int8")# ?? should be uint?
+            arrtypes[vtk.VTK_SHORT]         = ("SHORT",          "np.int16")
+            arrtypes[vtk.VTK_INT]           = ("INT",            "np.int32")
+            arrtypes[vtk.VTK_LONG]          = ("LONG",           "") # ??
+            arrtypes[vtk.VTK_LONG_LONG]     = ("LONG_LONG",      "np.int64")
+            arrtypes[vtk.VTK_FLOAT]         = ("FLOAT",          "np.float32")
+            arrtypes[vtk.VTK_DOUBLE]        = ("DOUBLE",         "np.float64")
+            arrtypes[vtk.VTK_SIGNED_CHAR]   = ("SIGNED_CHAR",    "np.int8")
+            arrtypes[vtk.VTK_ID_TYPE]       = ("ID",             "np.int64")
+
             printc(tab + "    scalar mode:", c="g", bold=1, end=" ")
             printc(mapper.GetScalarModeAsString(),
-                          '  coloring =', mapper.GetColorModeAsString(), c="g", bold=0)
+                   '  coloring =', mapper.GetColorModeAsString(), c="g", bold=0)
 
             printc(tab + "   active array: ", c="g", bold=1, end="")
             if ptdata.GetScalars():
@@ -1005,21 +1017,22 @@ def printInfo(obj):
                 if name and ptdata.GetArray(i):
                     printc(tab + "     point data: ", c="g", bold=1, end="")
                     try:
-                        tt = arrtypes[ptdata.GetArray(i).GetDataType()]
+                        tt, nptt = arrtypes[ptdata.GetArray(i).GetDataType()]
                     except:
-                        tt = str(ptdata.GetArray(i).GetDataType())
+                        tt = "VTKTYPE"+str(ptdata.GetArray(i).GetDataType())
+                        nptt = ""
                     ncomp = str(ptdata.GetArray(i).GetNumberOfComponents())
-                    printc("name=" + name, "("+ncomp+" "+tt+"),", c="g", bold=0, end="")
+                    printc("name=" + name, "("+ncomp+" "+tt+", "+nptt+"),", c="g", bold=0, end="")
                     rng = ptdata.GetArray(i).GetRange()
-                    printc(" range=(" + precision(rng[0],4) + ',' +
-                                            precision(rng[1],4) + ')', c="g", bold=0)
+                    printc(" range=(" + precision(rng[0],3) + ',' +
+                                        precision(rng[1],3) + ')', c="g", bold=0)
 
             for i in range(cldata.GetNumberOfArrays()):
                 name = cldata.GetArrayName(i)
                 if name and cldata.GetArray(i):
                     printc(tab + "      cell data: ", c="g", bold=1, end="")
                     try:
-                        tt = arrtypes[cldata.GetArray(i).GetDataType()]
+                        tt, nptt = arrtypes[cldata.GetArray(i).GetDataType()]
                     except:
                         tt = str(cldata.GetArray(i).GetDataType())
                     ncomp = str(cldata.GetArray(i).GetNumberOfComponents())
@@ -1601,8 +1614,8 @@ def makeTicks(x0, x1, N, labels=None, digits=None):
 
 def gridcorners(i, nm, size, margin=0, flipy=True):
     """
-    Compute the 2 corners of the i-th box of a grid n*m.
-    The bottom-left square is number 1.
+    Compute the 2 corners coordinates of the i-th box in a grid of shape n*m.
+    The top-left square is square number 1.
 
     Parameters
     ----------
@@ -1675,7 +1688,7 @@ def vedo2trimesh(mesh):
     lut = mesh.mapper().GetLookupTable()
 
     tris = mesh.faces()
-    carr = mesh.getCellArray('CellIndividualColors')
+    carr = mesh.celldata['CellIndividualColors']
     ccols = None
     if carr is not None and len(carr)==len(tris):
         ccols = []
@@ -1685,7 +1698,7 @@ def vedo2trimesh(mesh):
         ccols = np.array(ccols, dtype=np.int16)
 
     points = mesh.points()
-    varr = mesh.getPointArray('VertexColors')
+    varr = mesh.pointdata['VertexColors']
     vcols = None
     if varr is not None and len(varr)==len(points):
         vcols = []
@@ -1773,12 +1786,12 @@ def _vedo2meshlab(vmesh):
                   v_normals_matrix=vmesh.normals(cells=False, compute=False),
                   f_normals_matrix=vmesh.normals(cells=True, compute=False),
     )
-    parr = vmesh.getPointArray()
+    parr = vmesh.pointdata[0]
     if parr is not None:
         m.vertex_quality_array(parr)
     carr = vmesh.getCellArray()
     if carr is not None:
-        m.face_quality_array(carr)        
+        m.face_quality_array(carr)
     m.update_bounding_box()
     return m
 
@@ -1791,7 +1804,7 @@ def _meshlab2vedo(mmesh):
     mpoints, mcells = mmesh.vertex_matrix(), mmesh.face_matrix()
     pnorms = mmesh.vertex_normal_matrix()
     cnorms = mmesh.face_normal_matrix()
-        
+
     try:
         parr = mmesh.vertex_quality_array()
     except:
@@ -1799,13 +1812,13 @@ def _meshlab2vedo(mmesh):
     try:
         carr = mmesh.face_quality_array()
     except:
-        carr = None  
-    
+        carr = None
+
     if len(mcells):
         polydata = buildPolyData(mpoints, mcells)
     else:
         polydata = buildPolyData(mpoints, None)
-    
+
     if parr is not None:
         parr_vtk = numpy_to_vtk(parr)
         parr_vtk.SetName("MeshLabQuality")
@@ -1813,7 +1826,7 @@ def _meshlab2vedo(mmesh):
         if x1-x0:
             polydata.GetPointData().AddArray(parr_vtk)
             polydata.GetPointData().SetActiveScalars("MeshLabQuality")
-    
+
     if carr is not None:
         carr_vtk = numpy_to_vtk(carr)
         carr_vtk.SetName("MeshLabQuality")
@@ -1821,7 +1834,7 @@ def _meshlab2vedo(mmesh):
         if x1-x0:
             polydata.GetCellData().AddArray(carr_vtk)
             polydata.GetCellData().SetActiveScalars("MeshLabQuality")
-        
+
     if len(pnorms):
         polydata.GetPointData().SetNormals(numpy2vtk(pnorms))
     if len(cnorms):
