@@ -281,12 +281,12 @@ class BaseVolume:
         narray = utils.vtk2numpy(self._data.GetPointData().GetScalars()).reshape(narray_shape)
         narray = np.transpose(narray, axes=[2, 1, 0])
         return narray
-    
+
     def modified(self):
         """Use in conjunction with ``tonumpy()`` to update any modifications to the volume array"""
         self._data.GetPointData().GetScalars().Modified()
         return self
-    
+
     def dimensions(self):
         """Return the nr. of voxels in the 3 dimensions."""
         return np.array(self._data.GetDimensions())
@@ -890,6 +890,14 @@ class Volume(vtk.vtkVolume, BaseGrid, BaseVolume):
         always be grayscale i.e. the aggregated values are not passed through the
         color transfer function.
 
+    Example:
+
+        .. code-block:: python
+
+            from vedo import Volume
+            vol = Volume("path/to/mydata/rec*.bmp", c='jet', mode=1)
+            vol.show(axes=1)
+
     .. hint:: if a `list` of values is used for `alphas` this is interpreted
         as a transfer function along the range of the scalar.
 
@@ -941,19 +949,23 @@ class Volume(vtk.vtkVolume, BaseGrid, BaseVolume):
 
         ###################
         inputtype = str(type(inputobj))
-        #colors.printc('Volume inputtype', inputtype)
+
+        # colors.printc('Volume inputtype', inputtype, c='b')
 
         if inputobj is None:
             img = vtk.vtkImageData()
 
         elif utils.isSequence(inputobj):
 
-            if isinstance(inputobj[0], str): # scan sequence of BMP files
+            if isinstance(inputobj[0], str) and ".bmp" in inputobj[0].lower():
+                # scan sequence of BMP files
                 ima = vtk.vtkImageAppend()
                 ima.SetAppendAxis(2)
                 pb = utils.ProgressBar(0, len(inputobj))
                 for i in pb.range():
                     f = inputobj[i]
+                    if "_rec_spr.bmp" in f:
+                        continue
                     picr = vtk.vtkBMPReader()
                     picr.SetFileName(f)
                     picr.Update()
@@ -970,11 +982,11 @@ class Volume(vtk.vtkVolume, BaseGrid, BaseVolume):
                     inputobj = np.array(inputobj)
 
                 if len(inputobj.shape)==1:
-                    varr = utils.numpy2vtk(inputobj, dtype=np.float)
+                    varr = utils.numpy2vtk(inputobj, dtype=float)
                 else:
                     if len(inputobj.shape)>2:
                         inputobj = np.transpose(inputobj, axes=[2, 1, 0])
-                    varr = utils.numpy2vtk(inputobj.ravel(order='F'), dtype=np.float)
+                    varr = utils.numpy2vtk(inputobj.ravel(order='F'), dtype=float)
                 varr.SetName('input_scalars')
 
                 img = vtk.vtkImageData()
@@ -985,7 +997,8 @@ class Volume(vtk.vtkVolume, BaseGrid, BaseVolume):
                         colors.printc("Error: must set dimensions (dims keyword) in Volume.", c='r')
                         raise RuntimeError()
                     img.SetDimensions(inputobj.shape)
-                img.GetPointData().SetScalars(varr)
+                img.GetPointData().AddArray(varr)
+                img.GetPointData().SetActiveScalars(varr.GetName())
 
                 #to convert rgb to numpy
                 #        img_scalar = data.GetPointData().GetScalars()
@@ -1162,9 +1175,9 @@ class Volume(vtk.vtkVolume, BaseGrid, BaseVolume):
 
         |read_volume2| |read_volume2.py|_
         """
-        if vmin is None: 
+        if vmin is None:
             vmin, _ = self._data.GetScalarRange()
-        if vmax is None: 
+        if vmax is None:
             _, vmax = self._data.GetScalarRange()
         self._alphaGrad = alphaGrad
         volumeProperty = self.GetProperty()
@@ -1335,16 +1348,17 @@ class VolumeSlice(vtk.vtkImageSlice, Base3DProp, BaseVolume):
                     inputobj = np.array(inputobj)
 
                 if len(inputobj.shape)==1:
-                    varr = utils.numpy2vtk(inputobj, dtype=np.float)
+                    varr = utils.numpy2vtk(inputobj, dtype=float)
                 else:
                     if len(inputobj.shape)>2:
                         inputobj = np.transpose(inputobj, axes=[2, 1, 0])
-                    varr = utils.numpy2vtk(inputobj.ravel(order='F'), dtype=np.float)
+                    varr = utils.numpy2vtk(inputobj.ravel(order='F'), dtype=float)
                 varr.SetName('input_scalars')
 
                 img = vtk.vtkImageData()
                 img.SetDimensions(inputobj.shape)
-                img.GetPointData().SetScalars(varr)
+                img.GetPointData().AddArray(varr)
+                img.GetPointData().SetActiveScalars(varr.GetName())
 
         elif "ImageData" in inputtype:
             img = inputobj
